# Copyright 2013-2024 Lawrence Livermore National Security, LLC and other
# Spack Project Developers. See the top-level COPYRIGHT file for details.
#
# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from spack.package import *


class Podman(MakefilePackage):
    """An optionally rootless and daemonless container engine: alias docker=podman"""

    homepage = "https://podman.io"
    url = "https://github.com/containers/podman/archive/refs/tags/v4.3.1.tar.gz"
    maintainers("alecbcs")

    license("Apache-2.0")

    version("5.3.2", sha256="e7d7abf2d4ecae7217af017a4199d555563721bf6c3ae52e68704ee8268c432b")
    version("5.0.3", sha256="271e915bef96f42737517fe4df6e92e07717a486d769ce50123890b3a6721908")
    with default_args(deprecated=True):
        version("4.5.1", sha256="ee2c8b02b7fe301057f0382637b995a9c6c74e8d530692d6918e4c509ade6e39")
        version("4.3.1", sha256="455c29c4ee78cd6365e5d46e20dd31a5ce4e6e1752db6774253d76bd3ca78813")
        version("3.4.7", sha256="4af6606dd072fe946960680611ba65201be435b43edbfc5cc635b2a01a899e6e")
        version("3.4.2", sha256="b0c4f9a11eb500b1d440d5e51a6c0c632aa4ac458e2dc0362f50f999eb7fbf31")

    depends_on("c", type="build")  # generated

    # See <https://github.com/containers/podman/issues/16996> for the
    # respective issue and the suggested patch
    # issue was fixed as of 4.4.0
    patch("markdown-utf8.diff", when="@4:4.3.1")

    variant(
        "network",
        default="netavark",
        multi=False,
        values=(
            "cni",
            conditional("netavark", when="@4:")
        ),
    )

    depends_on("go", type="build")
    depends_on("go-md2man", type="build")
    depends_on("pkgconfig", type="build")
    depends_on("cni-plugins", type="run", when="network=cni")
    depends_on("netavark", type="run", when="network=netavark")
    depends_on("conmon", type="run")
    depends_on("runc", type="run")
    depends_on("slirp4netns", type="run")
    depends_on("gpgme")
    depends_on("libassuan")
    depends_on("libgpg-error")
    depends_on("libseccomp")

    # ALCF config
    depends_on("fuse-overlayfs", type="run")

    def patch(self):
        defs = FileFilter("vendor/github.com/containers/common/pkg/config/default.go")
        config_go = FileFilter(
            "vendor/github.com/containers/common/pkg/config/config_{}.go".format(
                self.spec.platform
            )
        )

        # Prepend the provided runc executable to podman's built-in runc search path
        defs.filter('"runc": {', '"runc": {' + '"{0}",'.format(self.spec["runc"].prefix.sbin.runc))
        # Prepend the provided conmon executable to podman's built-in conmon search path
        defs.filter(
            r"ConmonPath = \[\]string{",
            "ConmonPath = []string{"
            + '\n        "{0}",'.format(self.spec["conmon"].prefix.bin.conmon),
        )
        if self.spec.satisfies("network=cni"):
                # Prepend the provided cni-plugins directory to the cni-plugin search path
                defs.filter(
                    r"DefaultCNIPluginDirs = \[\]string{",
                    "DefaultCNIPluginDirs = []string{"
                    + '\n        "{0}",'.format(self.spec["cni-plugins"].prefix.bin),
                )
                # Set the default path for slirp4netns to the provided slirp4netns executable
                defs.filter(
                    "cniConfig := _cniConfigDir",
                    "cniConfig := _cniConfigDir"
                    + '\n        defaultEngineConfig.NetworkCmdPath = "{0}"'.format(
                	self.spec["slirp4netns"].prefix.bin.slirp4netns
                    ),
                )
        if self.spec.satisfies("network=netavark"):
                # Prepend the provided netavark directory to the netavark plugin search path
                defs.filter(
                    r"DefaultNetavarkPluginDirs = \[\]string{",
                    "DefaultNetavarkPluginDirs = []string{"
                    + '\n        "{0}",'.format(self.spec["netavark"].prefix.bin),
                )
                # Prepend netavark libexec/podman directory to default helper binary search path
                config_go.filter(
                    r"defaultHelperBinariesDir = \[\]string{",
                    "defaultHelperBinariesDir = []string{"
                    + '\n    "{0}",'.format(self.spec["netavark"].prefix.libexec.podman)
                )

        # modify `const` paths to /etc here since `const` can't be changed at build time
        # containers.conf
        config_go.filter(
            r"OverrideContainersConfig =.*",
            'OverrideContainersConfig = "{}/etc/containers/containers.conf"'.format(
                self.spec.prefix,
            ),
        )
        # seccomp.json
        defs.filter(
            r"SeccompOverridePath =.*",
            'SeccompOverridePath = "{}/etc/containers/seccomp.json"'.format(
                self.spec.prefix,
            ),
        )


    def setup_build_environment(self, env):

        env.set(
            'EXTRA_LDFLAGS',
            (
                # mounts.conf
                " -X github.com/containers/common.subscriptions.OverrideMountsFile={0}/etc/containers/mounts.conf"
                # policy.json
                " -X github.com/containers/image/v5/signature.systemDefaultPolicyPath={0}/etc/containers/policy.json"
                # registries.conf(.d)
                " -X github.com/containers/image/v5/sysregistriesv2.systemRegistriesConfPath={0}/etc/containers/registries.conf"
                " -X github.com/containers/image/v5/sysregistriesv2.systemRegistriesConfDirPath={0}/etc/containers/registries.conf.d"
                ## registries.d (docker)
                #" -X github.com/containers/image/v5/docker.systemRegistriesDirPath={0}/etc/containers/registries.d"
                # storage.conf
                " -X github.com/containers/storage/types.defaultOverrideConfigFile={0}/etc/containers/storage.conf"
            ).format(self.prefix)
        )

        env.set('ETCDIR', '{}/etc'.format(self.prefix))
        tags = "seccomp exclude_graphdriver_devicemapper exclude_graphdriver_btrfs"
        if self.spec.satisfies("network=cni"):
            tags += " cni"
        env.set('BUILDTAGS', tags)


    def install(self, spec, prefix):
        make("install", "PREFIX=" + prefix)
        # Install an initial etc/containers/policy.json (configured in prefix above)
        mkdirp(prefix.etc.containers)
        install("test/policy.json", prefix.etc.containers)
        # Cleanup directory trees which are created as part of the go build process
        remove_linked_tree(prefix.src)
        remove_linked_tree(prefix.pkg)
